import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { Badge, Button, IconButton, ListItemDecorator, Menu, MenuItem, Sheet, Typography, } from "@mui/joy";
import { Box, Dialog, DialogActions, Grow, Tooltip, useMediaQuery, useTheme, } from "@mui/material";
import { useEffect, useRef, useState } from "react";
import { useHotkeys } from "react-hotkeys-hook";
import { useNavigate } from "react-router-dom";
import { Searchable } from "./searchable";
import { formatLabel, genericIcons, useQuery } from "./utils";
export function NerdAdvancedSearch({ definitions, open = false, setOpen, variant = "toolbar", buttonFaIcon = "fa-solid fa-magnifying-glass", buttonLabel = "Search", onSearch, onNewSearchPreset, searchMetadata, }) {
    const [_open, _setOpen] = useState(open);
    const [_variant, _setVariant] = useState(variant ?? localStorage.getItem("searchVariant"));
    const [_advancedMode, _setAdvancedMode] = useState(false);
    const [moreAnchorEl, setMoreAnchorEl] = useState(null);
    const theme = useTheme();
    const sm = useMediaQuery(theme.breakpoints.down("sm"));
    const [addFieldAnchorEl, setAddFieldAnchorEl] = useState(null);
    const navigate = useNavigate();
    const _definitions = _advancedMode && searchMetadata ? searchMetadata : definitions;
    const query = useQuery();
    const filtersString = query.get("filters");
    const filters = filtersString ? JSON.parse(filtersString) : [];
    const ref = useRef();
    useEffect(() => {
        if (sm) {
            _setVariant("dialog");
        }
        else {
            _setVariant("toolbar");
        }
    }, [sm]);
    useEffect(() => {
        localStorage.setItem("searchVariant", _variant);
    }, [_variant]);
    useEffect(() => {
        if (setOpen)
            setOpen(_open);
        if (_open &&
            _variant === "toolbar" &&
            filters.length === 0 &&
            ref.current) {
            setAddFieldAnchorEl(ref?.current);
        }
    }, [_open, ref]);
    useEffect(() => {
        if (open != undefined)
            _setOpen(open);
    }, [open]);
    const go = () => {
        if (_variant === "dialog")
            _setOpen(false);
        query.set("offset", "0");
        navigate({ search: query.toString() });
        if (onSearch)
            onSearch({
                filters: query.get("filters"),
                offset: query.get("offset") ?? 0,
            });
    };
    useHotkeys(`enter`, go);
    const clearAll = () => {
        query.delete("filters");
        query.set("offset", "0");
        navigate({ search: query.toString() });
        if (onSearch)
            onSearch(undefined);
    };
    const handleAddField = (s) => () => {
        setAddFieldAnchorEl(null);
        filters.push({
            name: s.name,
            comparison: "eq",
            value: s.type === "bool" ? true : "",
        });
        query.set("filters", JSON.stringify(filters));
        navigate({ search: query.toString() });
    };
    const goButton = onSearch ? (_jsx(Tooltip, { title: "Enter", children: _jsx(Button, { endDecorator: _jsx("i", { className: "fa-sharp fa-solid fa-arrow-right" }), variant: "soft", size: "sm", sx: { background: _variant === "toolbar" ? "white" : undefined }, onClick: go, children: "Go" }) })) : ([]);
    const closeButton = (_jsx(Tooltip, { title: "Close Search", children: _jsx(IconButton, { size: "sm", variant: "solid", onClick: () => _setOpen(false), children: _jsx("i", { className: "fa-solid fa-xmark" }) }) }));
    const moreOptionsButton = (_jsxs(_Fragment, { children: [_jsx(IconButton, { size: "sm", variant: "solid", onClick: (e) => setMoreAnchorEl(e.currentTarget), children: _jsx("i", { className: "fa-solid fa-ellipsis-vertical" }) }), _jsxs(Menu, { anchorEl: moreAnchorEl, open: Boolean(moreAnchorEl), onClose: () => setMoreAnchorEl(null), children: [onNewSearchPreset ? (_jsxs(MenuItem, { onClick: () => {
                            onNewSearchPreset();
                            setMoreAnchorEl(null);
                        }, children: [_jsx(ListItemDecorator, { sx: { color: "inherit" }, children: _jsx("i", { className: "fa-solid fa-download" }) }), "Save as preset"] })) : ([]), _jsxs(MenuItem, { onClick: () => {
                            setMoreAnchorEl(null);
                            _setVariant((s) => (s === "dialog" ? "toolbar" : "dialog"));
                        }, children: [_jsx(ListItemDecorator, { sx: { color: "inherit" }, children: _jsx("i", { className: _variant === "toolbar"
                                        ? "fa-solid fa-up-right-and-down-left-from-center"
                                        : "fa-solid fa-down-left-and-up-right-to-center" }) }), _variant === "toolbar" ? "Expand" : "Reduce"] }), (searchMetadata?.length ?? 0) > 0 ? (_jsx(Tooltip, { title: "Uses Search Metadata", children: _jsxs(MenuItem, { onClick: () => {
                                setMoreAnchorEl(null);
                                _setAdvancedMode((s) => !s);
                            }, children: [_jsx(ListItemDecorator, { sx: { color: "inherit" }, children: _jsx("i", { className: "fa-solid fa-hammer" }) }), _advancedMode ? "Turn Dev Mode off" : "Dev Mode"] }) })) : ([]), _jsxs(MenuItem, { onClick: () => {
                            setMoreAnchorEl(null);
                            clearAll();
                        }, children: [_jsx(ListItemDecorator, { sx: { color: "inherit" }, children: _jsx("i", { className: "fa-solid fa-trash" }) }), "Clear all"] })] })] }));
    console.log(searchMetadata);
    const addFieldList = (_jsx(Menu, { placement: "bottom-start", anchorEl: addFieldAnchorEl, open: Boolean(addFieldAnchorEl), onClose: () => setAddFieldAnchorEl(null), sx: {
            maxHeight: 400,
            overflow: "scroll",
            width: sm ? "calc(100% - 32px)" : undefined,
        }, children: _definitions?.map((s) => (_jsxs(MenuItem, { onClick: handleAddField(s), children: [_jsx(ListItemDecorator, { sx: { color: "inherit" }, children: _jsx("i", { className: s.faIcon ?? genericIcons[s.type] }) }), formatLabel(s.label)] }, s.id))) }));
    const addFieldIconButton = (_jsxs(_Fragment, { children: [_jsx(Tooltip, { title: "Add a new field", children: _jsx(IconButton, { size: "sm", onClick: (e) => setAddFieldAnchorEl(e.currentTarget), children: _jsx("i", { className: "fa-sharp fa-solid fa-plus" }) }) }), addFieldList] }));
    const toolbarVariant = (_jsx(Grow, { in: true, children: _jsxs(Sheet, { sx: {
                p: 1,
                borderRadius: 4,
                display: "flex",
                gap: 1,
                justifyContent: "space-between",
                position: "absolute",
                width: "100%",
                height: "100%",
                left: 0,
                top: 0,
                zIndex: 99,
            }, color: "primary", variant: "solid", children: [_jsxs(Box, { className: "noScrollBar", sx: {
                        display: "flex",
                        gap: 1,
                        flexGrow: 1,
                        overflowY: "hidden",
                        overflowX: "scroll",
                        scrollbarWidth: "none",
                        width: 0,
                    }, children: [closeButton, filters.map((f, i) => (_jsx(Box, { sx: { flexShrink: 0, display: "flex" }, children: _jsx(Searchable, { index: i, definition: [..._definitions, ...(searchMetadata ?? [])]?.find((m) => m.name === f.name), onDelete: (i) => console.log(i) }) }, f.name))), _jsx(Sheet, { variant: "solid", color: "primary", sx: {
                                position: "sticky",
                                right: 0,
                                zIndex: 99,
                                gap: 1,
                                display: "flex",
                            }, children: addFieldIconButton })] }), _jsxs(Box, { sx: { display: "flex", gap: 1 }, children: [goButton, moreOptionsButton] })] }) }));
    const dialogVariant = (_jsxs(Dialog, { open: true, onClose: () => _setOpen(false), maxWidth: !sm ? "sm" : undefined, fullWidth: true, fullScreen: sm, children: [_jsxs(Sheet, { color: "primary", variant: "solid", sx: {
                    p: 2,
                    color: "white",
                    display: "flex",
                    flexDirection: "column",
                    gap: 1,
                    flex: sm ? 1 : undefined,
                }, children: [_jsxs(Box, { sx: { display: "flex", justifyContent: "space-between" }, children: [_jsx(Typography, { level: "h6", sx: { color: "inherit" }, startDecorator: _jsx("i", { className: "fa-solid fa-magnifying-glass" }), children: "Search" }), moreOptionsButton] }), _jsxs(Typography, { level: "body2", sx: { color: "inherit" }, children: ["Click on the plus button to add a new field. Construct your query. Click on ", _jsx("b", { children: "Go" }), " when you are done."] }), _jsx(Box, { sx: {
                            display: "flex",
                            flexDirection: "column",
                            gap: 1,
                            alignItems: "stretch",
                        }, children: filters.map((f, i) => (_jsx(Searchable, { index: i, definition: _definitions?.find((m) => m.name === f.name), onDelete: (i) => console.log(i) }, f.name))) }), _jsx(Box, { sx: { display: "flex", justifyContent: "center" }, children: addFieldIconButton })] }), _jsxs(DialogActions, { children: [_jsx(Button, { size: "sm", onClick: () => _setOpen(false), variant: "soft", color: "neutral", children: "Close" }), goButton] })] }));
    return (_jsxs(Box, { ref: ref, children: [_jsx(Badge, { badgeContent: filters.length, size: "sm", color: "danger", badgeInset: "8%", children: sm ? (_jsx(IconButton, { variant: "solid", size: "sm", onClick: () => _setOpen((o) => !o), children: _jsx("i", { style: { fontSize: 14 }, className: buttonFaIcon }) })) : (_jsx(Button, { variant: "solid", size: "sm", onClick: () => _setOpen((o) => !o), endDecorator: !sm && filters.length ? (_jsx(Tooltip, { title: "Clear Preset", children: _jsx(Box, { sx: { p: 0.5 }, onClick: (e) => {
                                e.stopPropagation();
                                e.preventDefault();
                                clearAll();
                            }, children: _jsx("i", { style: { fontSize: 14, color: "white" }, className: "fa-solid fa-xmark" }) }) })) : undefined, startDecorator: _jsx("i", { style: { fontSize: 14 }, className: buttonFaIcon }), children: buttonLabel })) }), _open ? (_jsxs(_Fragment, { children: [_variant === "toolbar" ? toolbarVariant : [], _variant === "dialog" ? dialogVariant : []] })) : ([])] }));
}
