import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { Menu, MenuItem, Sheet, TextField, Typography, useTheme, } from "@mui/joy";
import { Box, Divider, InputBase, Tooltip } from "@mui/material";
import { useState } from "react";
import { useNavigate } from "react-router-dom";
import { formatLabel, genericIcons, getTextWidth, useQuery } from "./utils";
export function Searchable({ index, definition }) {
    const navigate = useNavigate();
    const query = useQuery();
    const filtersString = query.get("filters");
    const filters = filtersString ? JSON.parse(filtersString) : [];
    const filter = filters[index];
    const onFilterValueChange = (value) => {
        if (value != undefined) {
            filters[index].value = value;
            onChange(filters);
        }
    };
    const onFilterComparisonChange = (comparison) => {
        if (comparison != undefined) {
            filters[index].comparison = comparison;
            onChange(filters);
        }
    };
    const onChange = (newFilters) => {
        query.set("filters", JSON.stringify(newFilters));
        navigate({ search: query.toString() });
    };
    const handleRemoveField = () => {
        if (index >= 0) {
            filters.splice(index, 1);
            if (filters.length) {
                query.set("filters", JSON.stringify(filters));
            }
            else {
                query.delete("filters");
            }
            navigate({ search: query.toString() });
        }
    };
    const getInteractor = () => {
        if (!definition)
            return _jsx(Typography, { children: filter.value });
        if (definition.renderInput)
            return definition.renderInput(filter.value, onFilterValueChange);
        switch (definition.type) {
            case "bool":
                return (_jsx(Comparison, { type: definition.type, value: filter.value ? "eq" : "neq", postfix: formatLabel(definition.label).toLowerCase(), onChange: (v) => {
                        onFilterValueChange(v === "eq");
                    } }));
            case "int":
                return (_jsxs(Box, { sx: { display: "flex", alignItems: "center" }, children: [_jsx(Typography, { color: "neutral", level: "body2", children: formatLabel(definition.label) }), _jsx(Comparison, { type: definition.type, value: filter.comparison, onChange: (value) => {
                                onFilterComparisonChange(value);
                            } }), _jsx(TextField, { placeholder: "Type here...", variant: "plain", autoFocus: true, type: "number", size: "sm", value: `${filter.value}`, onChange: (e) => {
                                onFilterValueChange(parseInt(e.target.value));
                            } })] }));
            case "string":
                return (_jsxs(Box, { sx: { display: "flex", alignItems: "center" }, children: [_jsx(Typography, { color: "neutral", level: "body2", children: formatLabel(definition.label) }), _jsx(Comparison, { type: definition.type, value: filter.comparison, onChange: (value) => {
                                onFilterComparisonChange(value);
                            } }), _jsx(InputBase, { placeholder: "Type here...", autoFocus: true, value: `${filter.value}`, sx: {
                                fontSize: 14,
                                width: getTextWidth(`${filter.value}`),
                                minWidth: `${filter.value}`.length === 0 ? 80 : undefined,
                            }, onChange: (e) => {
                                onFilterValueChange(e.target.value);
                            } })] }));
            case "date":
                return (_jsxs(Box, { sx: { display: "flex", alignItems: "center" }, children: [_jsx(Typography, { color: "neutral", level: "body2", children: formatLabel(definition.label) }), _jsx(Comparison, { type: definition.type, value: filter.comparison, onChange: (value) => {
                                onFilterComparisonChange(value);
                            } }), _jsx(TextField, { placeholder: "Type here...", variant: "plain", autoFocus: true, size: "sm", value: `${filter.value}`, type: "date", onChange: (e) => {
                                onFilterValueChange(e.target.value);
                            } })] }));
            default:
                return _jsx("div", {});
        }
    };
    let faIcon = "";
    if (definition) {
        faIcon = genericIcons[definition.type];
        if (definition.faIcon) {
            faIcon = definition.faIcon;
        }
        if (definition.getFaIcon) {
            faIcon = definition.getFaIcon(filter.value);
        }
    }
    return (_jsxs(Box, { sx: {
            background: "white",
            borderRadius: "8px",
            display: "flex",
            alignItems: "center",
            color: "text.primary",
            p: 1,
            gap: 1,
            height: 32,
        }, children: [definition ? (_jsx(Typography, { level: "body2", sx: { display: "flex", gap: 1 }, children: _jsx("i", { className: faIcon }) })) : ([]), _jsx(Divider, { orientation: "vertical" }), _jsx(Box, { sx: { flexGrow: 1 }, children: getInteractor() }), _jsx(Tooltip, { title: "Remove this field", children: _jsx("i", { style: { color: "inherit", cursor: "pointer" }, className: "fa-solid fa-xmark", onClick: handleRemoveField }) })] }));
}
function Comparison({ type, value, onChange, prefix, postfix, }) {
    const theme = useTheme();
    const [anchorEl, setAnchorEl] = useState(null);
    const config = {
        string: [
            { comparison: "eq", title: "is" },
            { comparison: "neq", title: "is not" },
            { comparison: "like", title: "is like" },
        ],
        bool: [
            { comparison: "eq", title: "is" },
            { comparison: "neq", title: "is not" },
        ],
        date: [
            { comparison: "eq", title: "is same as" },
            { comparison: "neq", title: "is not" },
            { comparison: "gte", title: "is after" },
            { comparison: "lte", title: "is before" },
        ],
        int: [
            { comparison: "eq", title: "is" },
            { comparison: "neq", title: "is not" },
            { comparison: "gt", title: "is greater than" },
            { comparison: "gte", title: "is greater or equal than" },
            { comparison: "lt", title: "is less than" },
            { comparison: "lte", title: "is less or equal than" },
        ],
    };
    const getOptions = () => {
        const JSX = [];
        for (const key in config[type]) {
            if (Object.prototype.hasOwnProperty.call(config[type], key)) {
                const e = config[type][key];
                JSX.push(_jsxs(MenuItem, { onClick: () => {
                        onChange(e.comparison);
                        setAnchorEl(null);
                    }, children: [prefix, " ", e.title, " ", postfix] }, e.comparison));
            }
        }
        return JSX;
    };
    return (_jsxs(_Fragment, { children: [_jsx(Tooltip, { title: "Change comparison", children: _jsxs(Sheet, { onClick: (e) => setAnchorEl(e.currentTarget), sx: {
                        p: 1,
                        color: "text.primary",
                        cursor: "pointer",
                        background: "rgba(155,155,155,0.0)",
                        transition: "background .3s",
                        fontSize: 14,
                        "&:hover": {
                            background: "rgba(155,155,155,0.15)",
                        },
                    }, children: [prefix, " ", _jsx("b", { style: { color: theme.palette.primary[600] }, children: config[type].find((e) => e.comparison === value)?.title }), " ", postfix, " ", _jsx("i", { style: { fontSize: 11 }, className: "fa-regular fa-chevron-down" })] }) }), _jsx(Menu, { anchorEl: anchorEl, open: Boolean(anchorEl), onClose: () => {
                    setAnchorEl(null);
                }, children: getOptions() })] }));
}
