import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { useCallback, useEffect, useState, useRef, useMemo, } from "react";
import { DataGridPro, GridToolbarContainer, GridToolbarFilterButton, GridLinkOperator, GridActionsCellItem, } from "@mui/x-data-grid-pro";
import CustomPagination from "../nerdPagination/nerdPagination";
import { Button, IconButton, Tooltip } from "@mui/material";
import { Delete, SaveAlt } from "@mui/icons-material";
import { operatorsByType } from "./operatorsByType";
import { useTranslation } from "react-i18next";
import nerdUII18n from "../i18n";
/**
 * NerdDataGrid component.
 *
 * @param {propsType} props NerdDataGrid props.
 * @returns {ReactElement} DataGrid wrapper with server-side filters.
 */
export default function NerdDataGrid(props) {
    const { columnsVisibility, columns, queryParams, setQueryparams, getRows, culumnsMatchingFilterName, hidePagination, hideFilters, defaultSelectRowID, createNewButton, requestPending, initialRowsPerPage, editable, updateRow, deleteRow, deleteSelectedRows, } = props;
    const rows = props.rows;
    const rowPageOptions = [10, 50, 100];
    let limit;
    let offset;
    let filters;
    if (queryParams) {
        limit = queryParams["limit"];
        offset = queryParams["offset"];
        filters = queryParams["filters"];
    }
    const [rowsPerPage, setRowsPerPage] = useState(initialRowsPerPage ? initialRowsPerPage : 10);
    const { t } = useTranslation("common", { i18n: nerdUII18n });
    const [page, setPage] = useState(1);
    const [selectPages, setSelectPages] = useState([0]);
    const [disabledNext, setDisabledNext] = useState(false);
    const [rowCount, setRowCount] = useState(undefined);
    const [rowCountState, setRowCountState] = useState(rowCount || 0);
    const [queryObject, setQueryObject] = useState(culumnsMatchingFilterName
        ? {
            filters: filters ? filters : "",
            limit: Number(limit) ? Number(limit) : rowsPerPage,
            offset: Number(offset) ? Number(offset) : 0,
        }
        : {
            limit: Number(limit) ? Number(limit) : rowsPerPage,
            offset: Number(offset) ? Number(offset) : 0,
        });
    const [filterModel, setFilterModel] = useState({
        items: [],
        linkOperator: GridLinkOperator.And,
    });
    const [selectionModel, setSelectionModel] = useState(defaultSelectRowID &&
        rows.filter((row) => row["id"] === defaultSelectRowID).length > 0
        ? [defaultSelectRowID]
        : []);
    const prevSelectionModel = useRef(defaultSelectRowID ? [defaultSelectRowID] : []);
    useEffect(() => {
        if (getRows) {
            setRowCount(undefined);
            getRows(queryObject);
        }
    }, [queryObject]);
    // Check if API call ends and close loading
    useEffect(() => {
        setRowCount(rows.length);
        //restore selection model after page change
        setSelectionModel(prevSelectionModel.current);
        setCustomPagination();
    }, [rows]);
    useEffect(() => {
        try {
            if (culumnsMatchingFilterName &&
                filters &&
                culumnsMatchingFilterName.size > 0) {
                const filtersObject = JSON.parse(filters);
                const filterModelTmp = {
                    items: [],
                    linkOperator: GridLinkOperator.And,
                };
                Array.isArray(filtersObject) &&
                    filtersObject.forEach((filter, i) => {
                        let columnField = "";
                        for (const [key, value] of culumnsMatchingFilterName.entries()) {
                            if (value === filter.name)
                                columnField = key;
                        }
                        filterModelTmp.items.push({
                            id: i + 1,
                            columnField,
                            value: filter.value,
                            operatorValue: filter.comparison,
                        });
                    });
                setFilterModel(filterModelTmp);
            }
        }
        catch (error) {
            return;
        }
    }, [culumnsMatchingFilterName]);
    // Set URL query
    useEffect(() => {
        if (setQueryparams) {
            if (Number(offset) % Number(limit) !== 0 ||
                !rowPageOptions.includes(Number(limit))) {
                const limit = initialRowsPerPage || 10;
                setQueryObject({
                    ...queryObject,
                    limit,
                    offset: 0,
                });
                setPage(1);
                setRowsPerPage(limit);
            }
            else if (Number(limit) === queryObject.limit) {
                setPage(Number(offset) / Number(limit) + 1);
                setRowsPerPage(Number(limit));
            }
            setQueryparams(queryObject);
        }
    }, [offset, limit, queryObject]);
    useEffect(() => {
        setRowCountState((prevRowCountState) => rowCount !== undefined ? rowCount : prevRowCountState);
    }, [rowCount, setRowCountState]);
    /**
     * Custom MUI Grid Toolbar
     *
     * @returns {React.ReactElement} Mui Grid Toolbar
     */
    function CustomToolbar() {
        return (_jsxs(GridToolbarContainer, { children: [!hideFilters && _jsx(GridToolbarFilterButton, {}), props.apiExportEndpoint && (_jsx(Button, { sx: { fontSize: "0.8rem" }, variant: "text", startIcon: _jsx(SaveAlt, {}), onClick: () => {
                        const a = encodeURI(`${props.apiExportEndpoint}${"filters" in queryObject
                            ? `?filters=${queryObject.filters}`
                            : ""}`);
                        window.open(a, "_blank", "noopener,noreferrer");
                    }, children: t("components:nerdDataGrid.export") })), createNewButton, editable && (_jsx(Tooltip, { title: t("components:nerdDataGrid.delete_rows"), children: _jsx(IconButton, { color: "primary", onClick: () => {
                            const selectedRows = rows.filter((row) => selectionModel.find((id) => row.id === id));
                            deleteSelectedRows?.(selectedRows.map((row) => {
                                return row.id;
                            }));
                        }, children: _jsx(Delete, {}) }) }))] }));
    }
    const setCustomPagination = useCallback(() => {
        if (Number(offset) > 0 || Number(limit) > 0) {
            setSelectPages((prevState) => [
                ...Array.from(new Set(prevState)),
                ...Array(page)
                    .fill(0)
                    .map((_e, i) => i + 1),
            ]);
        }
    }, [limit, offset, page]);
    const onFilterChange = useCallback((newFilterModel) => {
        if (culumnsMatchingFilterName) {
            const filters = [];
            newFilterModel.items.forEach((item) => {
                const searchMetadataName = culumnsMatchingFilterName.get(item.columnField);
                if (item.operatorValue &&
                    item.value !== undefined &&
                    item.value !== "" &&
                    searchMetadataName) {
                    filters.push({
                        name: searchMetadataName,
                        comparison: item.operatorValue,
                        value: item.value,
                    });
                }
            });
            setPage(1); // Reset page to 1
            setSelectPages([0]); // Reset select pages
            setQueryObject({
                limit: queryObject.limit,
                offset: 0,
                filters: JSON.stringify(filters),
            });
            setFilterModel(newFilterModel);
        }
    }, [culumnsMatchingFilterName, queryObject.limit]);
    /**
     * Set new page value and change the query object
     *
     * @param {React.MouseEvent<HTMLButtonElement> | null} _event ChangePage
     * @param {number} newPage New page index
     */
    function handleChangePage(_event, newPage) {
        prevSelectionModel.current = selectionModel;
        setPage(newPage);
        if (newPage > 0) {
            setQueryObject({
                ...queryObject,
                offset: (newPage - 1) * Number(limit),
            });
        }
    }
    /**
     * @param {ChangeEvent<HTMLInputElement | HTMLTextAreaElement>} event - new rows page number selected
     */
    function handleChangeRowsPerPage(event) {
        const newPage = parseInt(event.target.value, 10);
        setPage(1); // Reset page to 1
        setSelectPages([0]); // Reset select pages
        // Set new query object
        setQueryObject((prevState) => ({
            ...prevState,
            limit: newPage,
            offset: 0,
        }));
    }
    /**
     *
     * @param {GridSelectionModel} ids Selected rows
     */
    function onSelectionModelChange(ids) {
        const selectedIDs = new Set(ids);
        const selectedRows = rows.filter((row) => selectedIDs.has(row.id));
        const prevSelectedIDs = new Set(prevSelectionModel.current);
        //Don't call onSelect if page changed
        if (ids.length === 0) {
            if (rows.filter((row) => prevSelectedIDs.has(row.id)).length > 0) {
                props.onSelect?.(selectedRows);
            }
        }
        else {
            props.onSelect?.(selectedRows);
        }
        setSelectionModel(ids);
    }
    const handleDeleteClick = (id) => () => {
        deleteRow?.(Number(id));
    };
    const processRowUpdate = (newRow) => {
        const updatedRow = { ...newRow, isNew: false };
        updateRow?.(updatedRow);
        return updatedRow;
    };
    const columnsWithOperators = useMemo(() => {
        const columnsWithOperators = columns.map((col) => {
            return {
                ...col,
                filterOperators: operatorsByType(col.type),
            };
        });
        if (editable) {
            columnsWithOperators.push({
                field: "actions",
                type: "actions",
                headerName: "Actions",
                width: 100,
                cellClassName: "actions",
                getActions: ({ id }) => {
                    return [
                        _jsx(GridActionsCellItem, { icon: _jsx(Delete, {}), label: t("global.delete"), onClick: handleDeleteClick(id), onKeyUp: (event) => {
                                if (event.key === "Enter")
                                    handleDeleteClick(id);
                            } }, "GridActionDelete"),
                    ];
                },
            });
        }
        return columnsWithOperators;
    }, [columns]);
    return (_jsx(DataGridPro, { ...props, pageSize: rowsPerPage, rowsPerPageOptions: rowPageOptions, onStateChange: (state) => {
            const { pagination: { rowCount, pageSize }, } = state;
            setDisabledNext(rowCount < pageSize);
        }, pagination: true, paginationMode: "server", rowCount: rowCountState, hideFooterPagination: hidePagination, components: {
            Toolbar: CustomToolbar,
            Pagination: CustomPagination,
        }, componentsProps: {
            filterPanel: { linkOperators: ["and"] },
            pagination: {
                rowsPerPage,
                handleChangeRowsPerPage,
                rowPageOptions,
                handleChangePage,
                page,
                selectPages,
                disabledNext,
            },
        }, onSelectionModelChange: onSelectionModelChange, rows: rows, columns: columnsWithOperators, loading: requestPending, filterMode: "server", onFilterModelChange: onFilterChange, initialState: {
            columns: {
                columnVisibilityModel: columnsVisibility,
            },
        }, filterModel: filterModel, selectionModel: selectionModel, ...(editable && {
            experimentalFeatures: {
                newEditingApi: true,
            },
            processRowUpdate: processRowUpdate,
            checkboxSelection: true,
            disableSelectionOnClick: true,
        }) }));
}
