import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import Box from "@mui/material/Box";
import Button from "@mui/material/Button";
import Dialog from "@mui/material/Dialog";
import DialogActions from "@mui/material/DialogActions";
import DialogContent from "@mui/material/DialogContent";
import DialogContentText from "@mui/material/DialogContentText";
import DialogTitle from "@mui/material/DialogTitle";
import Stack from "@mui/material/Stack";
import React from "react";
import BorderLinearProgress from "../progess/borderLinearProgress";
import { useTranslation } from "react-i18next";
import nerdUII18n from "../../i18n";
import { Check, Error, Info, Warning } from "@mui/icons-material";
/**
 *
 * @param {NerdNotificationProps & openHookAction} param0 Component props
 * @returns {JSX.Element} Intrusive nerdNotifier
 */
export function Intrusive({ title, subtitle, body, icon, actions, severity = "info", open, autoHideDuration = 5000, autohide = true, showIcon = true, onHandleClose, unstyled = true, actionPrimaryProps = {
    variant: "text",
    color: "primary",
}, actionSecondaryProps = {
    variant: "text",
    color: "inherit",
}, }) {
    const [progress, setProgress] = React.useState(0);
    const timer = React.useRef(0);
    const { t } = useTranslation("common", { i18n: nerdUII18n });
    const handleClose = (_event, reason) => {
        if (reason === "clickaway" ||
            reason === "escapeKeyDown" ||
            reason === "backdropClick") {
            return;
        }
        onHandleClose(false);
    };
    React.useEffect(() => {
        timer.current = window.setInterval(() => {
            setProgress((prevProgress) => prevProgress >= 100 ? 0 : prevProgress + 10);
        }, autoHideDuration / 10);
        return () => {
            clearInterval(timer.current);
        };
    }, []);
    React.useEffect(() => {
        if (progress >= 100 && autohide) {
            onHandleClose(false);
        }
    }, [progress, autohide, open]);
    const severityIcon = severityIcons(severity);
    return (_jsx(Dialog, { open: open, onClose: handleClose, "aria-labelledby": "alert-dialog-title", "aria-describedby": "alert-dialog-description", keepMounted: true, transitionDuration: 1000, classes: {
            paper: "NerdNotifierIntrusivePaper",
        }, children: _jsxs(Stack, { spacing: 1, children: [autohide && !actions && (_jsx(BorderLinearProgress, { variant: "determinate", value: progress, color: severity })), showIcon && (_jsx(Box, { sx: {
                        display: "flex",
                        marginTop: "1rem",
                        justifyContent: "center",
                    }, children: icon != undefined ? icon : severityIcon })), title && _jsx(DialogTitle, { id: "alert-dialog-title", children: title }), unstyled ? (_jsxs(DialogContent, { children: [subtitle && (_jsx(DialogContentText, { id: "alert-dialog-description", children: subtitle })), body] })) : (_jsxs(React.Fragment, { children: [subtitle, body] })), !autohide && (_jsxs(DialogActions, { children: [_jsx(Button, { onClick: handleClose, ...actionSecondaryProps, children: t("global.cancel") }), _jsx(Button, { onClick: handleClose, ...actionPrimaryProps, children: t("global.ok") })] }))] }) }));
}
/**
 *
 * @param {string} severityText Severity text
 * @returns {ReactElement | null} Severity Icon
 */
function severityIcons(severityText) {
    switch (severityText) {
        case "error":
            return _jsx(Error, { color: "error" });
        case "info":
            return _jsx(Info, { color: "info" });
        case "success":
            return _jsx(Check, { color: "success" });
        case "warning":
            return _jsx(Warning, { color: "warning" });
        default:
            return null;
    }
}
